@@ -16,7 +16,7 @@ You will need PIL/Pillow to use the screenshot features.
 from __future__ import absolute_import, division, print_function
 
 
-__version__ = '0.9.45'
+__version__ = '0.9.46'
 
 import sys, time, datetime, os, platform
 
@@ -76,6 +76,13 @@ except ImportError:
         raise Exception('PyAutoGUI was unable to import pyscreeze. Please install this module.')
     center = grab = locate = locateAll = locateAllOnScreen = locateCenterOnScreen = locateOnScreen = pixel = pixelMatchesColor = screenshot = couldNotImportPyScreeze
 
+try:
+    import mouseinfo
+    def mouseInfo():
+        mouseinfo.MouseInfoWindow()
+except ImportError:
+    def mouseInfo():
+        raise Exception('PyAutoGUI was unable to import mouseinfo. Please install this module.')
 
 def useImageNotFoundException(value=None):
     if value is None:
@@ -306,7 +313,9 @@ def size():
 
 
 def onScreen(x, y=None):
-    """Returns whether the given xy coordinates are on the screen or not.
+    """Returns whether the given xy coordinates are on the primary screen or not.
+
+    Note that this function doesn't work for secondary screens.
 
     Args:
       Either the arguments are two separate values, first arg for x and second
@@ -1026,9 +1035,18 @@ def _mouseMoveDrag(moveOrDrag, x, y, xOffset, yOffset, duration, tween=linear, b
             # A single step does not require tweening.
             time.sleep(sleep_amount)
 
-        _failSafeCheck()
         tweenX = int(round(tweenX))
         tweenY = int(round(tweenY))
+
+        # Moving the cursor to a fail-safe corner as part of the planned
+        # mouse movements shouldn't trigger the fail-safe. This may seem
+        # ridiculous, but remember that (tweenX, tweenY) is the *calculated*
+        # coordinate of where the mouse should be, while _failSafeCheck()
+        # uses the *actual position* of the mouse from position() to
+        # decide if it should raise the fail-safe exception.
+        if (tweenX, tweenY) not in FAILSAFE_POINTS:
+            _failSafeCheck()
+
         if moveOrDrag == 'move':
             platformModule._moveTo(tweenX, tweenY)
         elif moveOrDrag == 'drag':
@@ -1036,7 +1054,8 @@ def _mouseMoveDrag(moveOrDrag, x, y, xOffset, yOffset, duration, tween=linear, b
         else:
             raise NotImplementedError('Unknown value of moveOrDrag: {0}'.format(moveOrDrag))
 
-    _failSafeCheck()
+    if (tweenX, tweenY) not in FAILSAFE_POINTS:
+        _failSafeCheck()
 
 
 # Keyboard Functions
@@ -1225,16 +1244,21 @@ def _failSafeCheck():
 def displayMousePosition(xOffset=0, yOffset=0):
     """This function is meant to be run from the command line. It will
     automatically display the location and RGB of the mouse cursor."""
+    try:
+        runningIDLE = sys.stdin.__module__.startswith('idlelib')
+    except:
+        runningIDLE = False
+
     print('Press Ctrl-C to quit.')
     if xOffset != 0 or yOffset != 0:
         print('xOffset: %s yOffset: %s' % (xOffset, yOffset))
-    resolution = size()
     try:
         while True:
             # Get and print the mouse coordinates.
             x, y = position()
             positionStr = 'X: ' + str(x - xOffset).rjust(4) + ' Y: ' + str(y - yOffset).rjust(4)
-            if (x - xOffset) < 0 or (y - yOffset) < 0 or (x - xOffset) >= resolution[0] or (y - yOffset) >= resolution[1]:
+            if not onScreen(x - xOffset, y - yOffset) or sys.platform == 'darwin':
+                # Pixel color can only be found for the primary monitor, and also not on mac due to the screenshot having the mouse cursor in the way.
                 pixelColor = ('NaN', 'NaN', 'NaN')
             else:
                 pixelColor = pyscreeze.screenshot().getpixel((x, y)) # NOTE: On Windows & Linux, getpixel() returns a 3-integer tuple, but on macOS it returns a 4-integer tuple.
@@ -1242,7 +1266,13 @@ def displayMousePosition(xOffset=0, yOffset=0):
             positionStr += ', ' + str(pixelColor[1]).rjust(3)
             positionStr += ', ' + str(pixelColor[2]).rjust(3) + ')'
             sys.stdout.write(positionStr)
-            sys.stdout.write('\b' * len(positionStr))
+            if not runningIDLE:
+                # If this is a terminal, than we can erase the text by printing \b backspaces.
+                sys.stdout.write('\b' * len(positionStr))
+            else:
+                # If this isn't a terminal (i.e. IDLE) then we can only append more text. Print a newline instead and pause a second (so we don't send too much output).
+                sys.stdout.write('\n')
+                time.sleep(1)
             sys.stdout.flush()
     except KeyboardInterrupt:
         sys.stdout.write('\n')
